@@ -28,13 +28,15 @@ class Zmod:
     0
     """
 
-    def __init__(self, n: int, short: bool = True):
+    def __init__(self, n: int, short: bool = True, sharp: bool = False):
         if not isinstance(n, int) or n < 1:
             raise ValueError(f"{n} is not a positive integer.")
         self.n = n
+        self.n2 = n // 2
         self.isfield = None
         self.n_factors = {} # factoring of n
         self.short = short
+        self.sharp = sharp
         self.group_order = 0 # order of the multiplicative group
         self.factors = {}  # factoring of the multiplicative group order
 
@@ -161,9 +163,13 @@ class ZmodPoint:
         self.ring = ring
 
     def __repr__(self):
+        if self.ring.sharp:
+            x = self.sharp()
+        else:
+            x = self.x
         if self.ring.short:
-            return str(self.x)
-        return f"{self.x} (mod {self.ring.n})"
+            return str(x)
+        return f"{x} (mod {self.ring.n})"
 
     def __eq__(self, other):
         if not isinstance(other, self.__class__) or self.ring != other.ring:
@@ -239,7 +245,9 @@ class ZmodPoint:
         return NotImplemented
 
     def __abs__(self) -> int:
-        return abs(self.sharp())
+        if self.x <= self.ring.n2:
+            return self.x
+        return self.ring.n - self.x
 
     def bits(self) -> list:
         "Convert to a list of bits."
@@ -247,7 +255,7 @@ class ZmodPoint:
 
     def sharp(self):
         "Returns a symmetric (w.r.t. 0) representative."
-        if self.x <= self.ring.n // 2:
+        if self.x <= self.ring.n2:
             return self.x
         return self.x - self.ring.n
 
@@ -320,6 +328,18 @@ class ZmodPoint:
             roots.append((p**(j//2) * x) % pk)
         return self.ring(crt(roots, powers))
 
+    def solve(self, b):
+        "Find a solution `x` of the linear equation `self * x == b` in Z_n."
+        b = int(b)
+        if not self:
+            if b:
+                return None
+            return self.ring(0)
+        g = gcd(self.x, self.ring.n)
+        if b % g:
+            return None
+        return self.ring(pow(self.x // g, -1, self.ring.n) * (b // g))
+
     def is_generator(self):
         "Test if the point is a generator of the group Z_n^*."
         return self.ring.order() == self.order()
