@@ -55,21 +55,22 @@ def factorint(n: int, trial_bnd: int = 2500, verbose: int = 0) -> dict:
         prime_factors[-1] = 1
 
     def add_factors(m: int, mm: tuple) -> None:
+        "Delete m from remaining factors and add its factors mm to the appropriate dict."
         k = remaining_factors[m]
         del remaining_factors[m]
 
-        for m in mm:
-            if m in prime_factors:
-                prime_factors[m] += k
-            elif is_prime(m, trialdivision=False):
-                prime_factors[m] = k
+        for m_i in mm:
+            if m_i in prime_factors:
+                prime_factors[m_i] += k
+            elif is_prime(m_i, trialdivision=False):
+                prime_factors[m_i] = k
             else:
-                if m in remaining_factors:
-                    remaining_factors[m] += k
-                elif m in new_factors:
-                    new_factors[m] += k
+                if m_i in remaining_factors:
+                    remaining_factors[m_i] += k
+                elif m_i in new_factors:
+                    new_factors[m_i] += k
                 else:
-                    new_factors[m] = k
+                    new_factors[m_i] = k
 
     if verbose:
         print(f"Factoring: {n} ({floor(log10(n)) + 1} digits)")
@@ -92,7 +93,7 @@ def factorint(n: int, trial_bnd: int = 2500, verbose: int = 0) -> dict:
     if res:
         n, k = res
     else:
-        n, k = n, 1
+        k = 1
     if is_prime(n, trialdivision=False):
         prime_factors[n] = k
         return prime_factors
@@ -114,7 +115,7 @@ def factorint(n: int, trial_bnd: int = 2500, verbose: int = 0) -> dict:
     ]
     fermat_steps = 10
 
-    for round, parameters in enumerate(ECM_PARAMETERS):
+    for nr, parameters in enumerate(ECM_PARAMETERS):
         fermat_steps += 10
         B1, B2, num_curves = parameters
         p_max = isqrt(max(remaining_factors))+1
@@ -126,7 +127,7 @@ def factorint(n: int, trial_bnd: int = 2500, verbose: int = 0) -> dict:
         D = isqrt(B2)
 
         if verbose:
-            print(f"Round {round+1} (B1={B1})")
+            print(f"Round {nr+1} (B1={B1})")
 
         primes = sieve_eratosthenes(B1 - 1 + ((B2 - B1 + 1) // (2 * D) + 1) * 2 * D)
         pm1_parameters = tuple([10 * B1, B2] + list(_pm1_parameters(10 * B1, B2, primes=primes)))
@@ -135,14 +136,14 @@ def factorint(n: int, trial_bnd: int = 2500, verbose: int = 0) -> dict:
         methods = {_factor_fermat: "fmt", factor_pm1: "pm1", factor_ecm: "ecm"}  #, factor_qs: "qs"}
         while remaining_factors:
             new_factors = {}
-            for method in methods:
-                factors = list(remaining_factors)
+            for method, method_name in methods.items():
+                factors = list(remaining_factors) # make a copy to avoid a runtime error if the dict changes
                 for m in factors:
-                    if method == _factor_fermat:
+                    if method == _factor_fermat:  # pylint: disable=W0143
                         tmp = _factor_fermat(m, maxsteps=fermat_steps, verbose=max(0, verbose - 1))
-                    elif method == factor_pm1:
+                    elif method == factor_pm1:  # pylint: disable=W0143
                         tmp = factor_pm1(m, pm1_parameters=pm1_parameters, verbose=max(0, verbose - 1))
-                    elif method == factor_ecm:
+                    elif method == factor_ecm:  # pylint: disable=W0143
                         tmp = factor_ecm(m, ecm_parameters=ecm_parameters, verbose=max(0, verbose - 1))
                     else:
                         tmp = method(m, verbose=max(0, verbose - 1))
@@ -160,16 +161,18 @@ def factorint(n: int, trial_bnd: int = 2500, verbose: int = 0) -> dict:
                                     tmp3.append(x)
                         else:
                             tmp3 = [tmp, tmp2]
-                        if verbose > 0: print(f"Factors found ({methods[method]}): ", tmp3)
+                        if verbose > 0:
+                            print(f"Factors found ({method_name}): ", tmp3)
                         add_factors(m, tmp3)
             if not new_factors:
                 break
-            for m in new_factors:
+            for m, k in new_factors.items():
                 if m in remaining_factors:
-                    remaining_factors[m] += new_factors[m]
+                    remaining_factors[m] += k
                 else:
-                    remaining_factors[m] = new_factors[m]
-            if verbose > 1: print("Remaining: ", remaining_factors)
+                    remaining_factors[m] = k
+            if verbose > 1:
+                print("Remaining: ", remaining_factors)
 
         if len(remaining_factors) == 0:
             return prime_factors
@@ -190,7 +193,7 @@ def divisors(n:int, proper:bool = False) -> int:
     if n <= 1:
         return divisor_list
     facctordict= factorint(n)
-    primes = [p for p in facctordict]
+    primes = list(facctordict)
     nprimes = len(primes)
     multiplicites = [facctordict[p] for p in primes]
 
