@@ -1,6 +1,7 @@
-import pytest
-from random import randint, seed
-from math import gcd
+# pragma pylint: disable=C0114,C0116
+import pytest  # pylint: disable=W0611
+from random import randint, seed  # pylint: disable=C0411
+from math import gcd  # pylint: disable=C0411
 from kryptools import EC_Weierstrass
 seed(0)
 
